class StringSet {
  constructor(resourcesArr=[], keys=[]){
    this.resources = resourcesArr;
    this.keys = keys;
  }
  toXML(ordinal) {
    if (typeof ordinal === 'number') {
      return `<?xml version="1.0" encoding="utf-8"?>\n<resources>\n  ${this.resources.map(it => it.toXML(ordinal)).join('\n  ')}\n</resources>`;
    } else if (this.keys.indexOf(ordinal) !== -1) {
      const index = this.keys.indexOf(ordinal);
      return `<?xml version="1.0" encoding="utf-8"?>\n<resources>\n  ${this.resources.map(it => it.toXML(index)).join('\n  ')}\n</resources>`;
    }
  }
}

const indent = (str, spaces = 2) => (" ").repeat(spaces) + str;

class StringResource {
  constructor (name, ...values) {
    this.name = name;
    this.values = values
  }
  toXML(ordinal) {
    return `<string name="${this.name}">${this.values[ordinal]}</string>`;
  }
  static fromObj(obj) {
    let resources = [];
    for (const key in obj) {
      resources.push(new StringResource(key, ...obj[key]));
    }
    return resources;
  }
};

const generateCSV = csv => {
  const lines = csv.split('\n');
  let parsed = lines.map(line => line.split(','));
  const resources = parsed.slice(1).map(values => new StringResource(values[0], ...values));
  return new StringSet(resources, parsed[0]);
};

const flattenArr = arr => arr.reduce((sum, next) => sum.concat(next), []);
const flattenObject = obj => flattenArr(Object.values(obj).map(value => typeof value === 'object' ? flattenObject(value) : value));

const forEachRecursive = (object, callback) => {
  for (const key in object) {
    const value = object[key];
    if (typeof value === 'object') return forEachRecursive(value, callback);
    callback(key, value);
  }
};

const generateJSON = (obj, depth = 0) => {
  const partials = {};
  let keys = [];
  let parsedObj = obj;
  if (typeof obj === 'string') parsedObj = JSON.parse(obj);
  for (const key in parsedObj) {
    keys.push(key);
    forEachRecursive(parsedObj[key], (k,v) => {
      partials[k] = partials[k] || [];
      partials[k].push(v);
    });
  };
  return new StringSet(StringResource.fromObj(partials), keys);
};

module.exports = {
  json: generateJSON,
  csv: generateCSV,
};
